package Tietorakenteet;

import java.util.ArrayList;

/**
 * @author MHo
 * @version 20.2.2020
 * Hiihtorekisterin jasenet, joka lisää ja poistaa jäseniä.
 */
public class Hiihdot {
    private String tiedostonNimi = "";
    private ArrayList<Hiihto> hiihdot = new ArrayList<Hiihto>();
    
    /**
     * Muodostaja
     */
    public Hiihdot(){
       //
    }
    
    
    
    /**
     * @param hiihto suoritus, joka lisätään rekisteriin
     */
    public void lisaa(Hiihto hiihto){
        hiihdot.add(hiihto);
    }
    
    
    /**
     * Palauttaa viitteen i:teen jäseneen.
     * @param i monennenko jäsenen viite halutaan
     * @return viite jäseneen, jonka indeksi on i
     * @throws IndexOutOfBoundsException jos i ei ole sallitulla alueella
     * @example
     * <pre name="test">
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * Hiihdot hiihdot = new Hiihdot();
     * hiihdot.anna(0) === eka;
     * hiihdot.anna(1) === toka;
     * </pre>  
     */
    public Hiihto anna(int i) throws IndexOutOfBoundsException {
        
        if (i < 0 || hiihdot.size() <= i)
            throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return hiihdot.get(i);
    }
    
    
    /**
     * Lukee hiihdot tiedostosta.
     * @param hakemisto tiedoston hakemisto
     * @throws SailoException jos lukeminen epäonnistuu
     */
    public void lueTiedostosta(String hakemisto) throws SailoException {
        tiedostonNimi = hakemisto + "/nimet.dat";
        throw new SailoException("Ei osata vielä lukea tiedostoa " + tiedostonNimi);
    }


    /**
     * Tallentaa hiihdon tiedostoon.
     * @throws SailoException jos talletus epäonnistuu
     */
    public void talleta() throws SailoException {
        throw new SailoException("Ei osata vielä tallentaa tiedostoa " + tiedostonNimi);
    }


    /**
     * @param id jasenen ID
     * @return tietyn jäsenen hiihdot listana
     * @example
     * <pre name="test">
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * Hiihto kolmas = new Hiihto(2);
     * Hiihdot hiihdot = new Hiihdot();
     * hiihdot.lisaa(eka);
     * hiihdot.lisaa(toka);
     * hiihdot.lisaa(kolmas);
     * hiihdot.getJasenenHiihdot(1) === "[eka, toka]";
     * hiihdot.getJasenenHiihdot(2) === "[kolmas]";
     * </pre>
     */
    public ArrayList<Hiihto> getJasenenHiihdot(int id) {
        ArrayList<Hiihto> jasenenHiihdot = new ArrayList<Hiihto>();
        for (int i = 0; i < hiihdot.size(); i++) {
            if (hiihdot.get(i).getJasenID() == id) jasenenHiihdot.add(hiihdot.get(i));
        }
        return jasenenHiihdot;
    }
    
    
    /**
     * @return rekisterin hiihtosuoritusten maara
     * @example
     * <pre name="test">
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * Hiihto kolmas = new Hiihto(3);
     * Hiihdot hiihdot = new Hiihdot();
     * hiihdot.lisaa(eka);
     * hiihdot.lisaa(toka);
     * hiihdot.lisaa(kolmas);
     * hiihdot.getHiihdot() === 3;
     * </pre>
     */
    public int getHiihdot() {
        return hiihdot.size();
    }
    
    
    /**
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Hiihdot hiihdot = new Hiihdot();

        Hiihto lenkki = new Hiihto(1), toinen = new Hiihto(2);

        try {
            hiihdot.lisaa(lenkki);
            hiihdot.lisaa(toinen);

            System.out.println("========== Hiihdot testi ==============");

            for (int i=0; i < hiihdot.getHiihdot(); i++) {
                Hiihto hiihto = hiihdot.anna(i);
                System.out.println("Hiihto nro: " + i);
                hiihto.tulosta(System.out);
            }

        } catch ( Exception ex ) {
            System.out.println(ex.getMessage());
        }
    }
}