package Tietorakenteet;

/**
 * @author MHo
 * @version 20.2.2020
 * Hiihtorekisterin jasenet, joka lisää ja poistaa jäseniä.
 */
public class Jasenet {
    private static int MAX_JASENIA = 5;
    private int lkm = 0;
    private String tiedostonNimi = "";
    private Jasen alkiot[] = new Jasen[MAX_JASENIA];
    //private ArrayList<Jasen> jasenisto = new ArrayList<Jasen>();
    
    
    /**
     * Muodostaja
     */
    public Jasenet(){
       //
    }
    
    
    /**
     * Lisää uuden jäsenen tietorakenteeseen.  Ottaa jäsenen omistukseensa.
     * @param jasen lisätäävän jäsenen viite.  Huom tietorakenne muuttuu omistajaksi
     * @example
     * <pre name="test">
     * Jasenet jasenet = new Jasenet();
     * Jasen aku1 = new Jasen(), aku2 = new Jasen();
     * jasenet.getLkm() === 0;
     * jasenet.lisaa(aku1); jasenet.getLkm() === 1;
     * jasenet.lisaa(aku2); jasenet.getLkm() === 2;
     * jasenet.lisaa(aku1); jasenet.getLkm() === 3;
     * jasenet.anna(0) === aku1;
     * jasenet.anna(1) === aku2;
     * jasenet.anna(2) === aku1;
     * jasenet.anna(1) == aku1 === false;
     * jasenet.anna(1) == aku2 === true;
     * jasenet.anna(3) === aku1; #THROWS IndexOutOfBoundsException 
     * jasenet.lisaa(aku1); jasenet.getLkm() === 4;
     * jasenet.lisaa(aku1); jasenet.getLkm() === 5;
     * </pre>
     */
    public void lisaa(Jasen jasen){
        if (lkm >= alkiot.length) {
        Jasen uusi[] = new Jasen[MAX_JASENIA+10];
        MAX_JASENIA = uusi.length;
           for (int i = 0; i < alkiot.length; i++) {
           uusi[i] = alkiot[i];
           }
           alkiot = uusi;
        }
        alkiot[lkm] = jasen;
        lkm++;
    }
    
    
    /**
     * Palauttaa viitteen i:teen jäseneen.
     * @param i monennenko jäsenen viite halutaan
     * @return viite jäseneen, jonka indeksi on i
     * @throws IndexOutOfBoundsException jos i ei ole sallitulla alueella  
     * @example
     * <pre name="test">
     * Jasen testi = new Jasen();
     * Jasen toinen = new Jasen();
     * Jasenet jasenet = new Jasenet();
     * jasenet.lisaa(testi);
     * jasenet.lisaa(toinen);
     * jasenet.anna(0) === testi;
     * jasenet.anna(1) === toinen;
     * jasenet.anna(8) === toinen; #THROWS IndexOutOfBoundsException
     * </pre>
     */
    public Jasen anna(int i) throws IndexOutOfBoundsException {
        
        if (i < 0 || lkm <= i)
            throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return alkiot[i];
    }
    
    
    /**
     * Lukee jäsenistön tiedostosta.  Kesken.
     * @param hakemisto tiedoston hakemisto
     * @throws SailoException jos lukeminen epäonnistuu
     */
    public void lueTiedostosta(String hakemisto) throws SailoException {
        tiedostonNimi = hakemisto + "/nimet.dat";
        throw new SailoException("Ei osata vielä lukea tiedostoa " + tiedostonNimi);
    }


    /**
     * Tallentaa jäsenistön tiedostoon.  Kesken.
     * @throws SailoException jos talletus epäonnistuu
     */
    public void talleta() throws SailoException {
        throw new SailoException("Ei osata vielä tallettaa tiedostoa " + tiedostonNimi);
    }


    /**
     * Palauttaa kerhon jäsenten lukumäärän
     * @return jäsenten lukumäärä
     * @example
     * <pre name="test">
     * Jasen eka = new Jasen();
     * Jasen toka = new Jasen();
     * Jasenet jasenet = new Jasenet();
     * jasenet.lisaa(eka);
     * jasenet.lisaa(toka);
     * jasenet.getLkm() === 2;
     * </pre>
     */
    public int getLkm() {
        return lkm;
    }
    
    
    /**
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Jasenet jasenet = new Jasenet();

        Jasen nokka = new Jasen(), raka = new Jasen();

        try {
            jasenet.lisaa(nokka);
            jasenet.lisaa(raka);

            System.out.println("========== Jäsenet testi ==============");

            for (int i=0; i<jasenet.getLkm(); i++) {
                Jasen jasen = jasenet.anna(i);
                System.out.println("Jäsen nro: " + i);
                jasen.tulosta(System.out);
            }

        } catch ( Exception ex ) {
            System.out.println(ex.getMessage());
        }
    }
}